@@ -10,7 +10,7 @@ from testmon.process_code import blob_to_checksums, checksums_to_blob
 from testmon.common import TestExecutions
 
 
-DATA_VERSION = 13
+DATA_VERSION = 14
 
 ChangedFileData = namedtuple(
     "ChangedFileData", "filename name method_checksums id failed"
@@ -359,7 +359,7 @@ class DB:  # pylint: disable=too-many-public-methods
                 duration FLOAT,
                 failed BIT,
                 forced BIT,
-                FOREIGN KEY({self._test_execution_fk_column()}) REFERENCES {self._test_execution_fk_table()}(id));
+                FOREIGN KEY({self._test_execution_fk_column()}) REFERENCES {self._test_execution_fk_table()}(id) ON DELETE CASCADE);
                 CREATE INDEX test_execution_fk_name ON test_execution ({self._test_execution_fk_column()}, test_name);
             """
 
@@ -394,7 +394,7 @@ class DB:  # pylint: disable=too-many-public-methods
             CREATE TABLE test_execution_file_fp (
                 test_execution_id INTEGER,
                 fingerprint_id INTEGER,
-                FOREIGN KEY(test_execution_id) REFERENCES test_execution(id),
+                FOREIGN KEY(test_execution_id) REFERENCES test_execution(id) ON DELETE CASCADE,
                 FOREIGN KEY(fingerprint_id) REFERENCES file_fp(id)
             );
             CREATE INDEX test_execution_file_fp_both ON test_execution_file_fp (test_execution_id, fingerprint_id);
@@ -403,7 +403,7 @@ class DB:  # pylint: disable=too-many-public-methods
                 suite_execution_id INTEGER,
                 filename TEXT,
                 fsha text,
-                FOREIGN KEY(suite_execution_id) REFERENCES suite_execution(id)
+                FOREIGN KEY(suite_execution_id) REFERENCES suite_execution(id) ON DELETE CASCADE
                 );
                 CREATE UNIQUE INDEX sefch_suite_id_filename_sha ON suite_execution_file_fsha(suite_execution_id, filename, fsha);
             """
@@ -659,7 +659,7 @@ class DB:  # pylint: disable=too-many-public-methods
                 ORDER BY id DESC
                 """,
                 (environment_name,),
-            ).fetchone()
+            ).fetchone()  # TODO to pick best environment of many with the same name, we would need to fetch all.
 
             if environment:
                 environment_id = environment["id"]
@@ -669,7 +669,7 @@ class DB:  # pylint: disable=too-many-public-methods
                 )
             else:
                 packages_changed = False
-            if not environment or packages_changed:
+            if not environment:
                 try:
                     cursor.execute(
                         """
@@ -694,6 +694,19 @@ class DB:  # pylint: disable=too-many-public-methods
                         (environment_name,),
                     ).fetchone()
                     environment_id = environment["id"]
+            elif packages_changed:
+                cursor.execute(
+                    """
+                    INSERT INTO environment (environment_name, system_packages, python_version)
+                    VALUES (?, ?, ?)
+                    """,
+                    (environment_name, system_packages, python_version),
+                )
+                new_environment_id = cursor.lastrowid
+                cursor.execute(
+                    "DELETE FROM environment WHERE id = ?", (environment_id,)
+                )
+                environment_id = new_environment_id
 
             return environment_id, packages_changed
 
